'=================================================================================================================
' Script to run through a BeatBuddy project folder and create MIDI Presets in the form of an importMIDI.txt file 
' for subsequent import to Bandhelper app
'
' Author: SiblingChris
' Version: 2.0
' Date: 7th Feb 2018
'
'=================================================================================================================
Option Explicit


'-----------------------------------------------------------------------------------------------------------------
' Class definitions for DrumSets, Tempos and Songs
'-----------------------------------------------------------------------------------------------------------------

' Drum Set class
Class DrumSet

	Public Id
	Public Description
	Public Name 
	Public Number	
	Public ControlChange(1) 
	Public RawMIDI
	Public MIDIChannel
	Public PresetName	
	
	' class constructor
	Public Default Function Init( DrumSetFileName, DrumSetDescription, Channel, Numbering )
	
		' Drum Set Id and Description as shown on the BeatBuddy menu
		Id = Left(DrumSetFileName, Len(DrumSetFileName) - 4)
		Description = DrumSetDescription
						
		' MIDI channel
		If Channel < 1 or Channel > 16 Then
			MIDIChannel = 1
		Else
			MIDIChannel = Channel
		End If
		
		' Drum Set name  
		Name = Trim( Split(Description, ".")(1) )		    
		
		' set CC-116 and value representing the number of the drumset in the BeatBuddy menu
		ControlChange(0) = 116
		ControlChange(1) = CInt(Split(Description, ".")(0))
		Number = ControlChange(1)
		
		' the preset name to be used in BandHelper
		PresetName = "D " & Right("0" & Trim(CStr(ControlChange(1))), 2) & " " & Name 
				
		' define the Raw MIDI Hex String for Bandhelper to send to the BeatBuddy to select this drumset
		RawMIDI = "B" & Hex(MIDIChannel - 1) & " " & _
		          Right("0" & Hex(ControlChange(0)), 2) & " " & _
		          Right("0" & Hex(ControlChange(1)), 2)
		          ' "74 " & Right("0" & Hex(Number), 2)
		
		' return the object
		Set Init = Me
		
	End Function
  	
End Class

' Tempo class
Class Tempo

	Public BPM
	Public TempoMSB(1)
	Public TempoLSB(1)
	Public NRPNTempoMSB(1)
	Public NRPNTempoLSB(1)
	Public DataINC(1)
	Public DataDEC(1)
	Public NRPNClearMSB(1)
	Public NRPNClearLSB(1)	
	Public ControlChange(1) 
	Public RawMIDI
	Public MIDIChannel
	Public PresetName
	
	' class constructor
	Public Default Function Init( beatsPerMinute, Channel, ChangeFunction )
		
		' loop counter
		Dim i
			
		If ChangeFunction <> "INC" and ChangeFunction <> "DEC" Then 		
			
			' bpm
			Select Case True
				Case beatsPerMinute < 40  and beatsPerMinute <> 0
					BPM = 40
				Case beatsPerMinute > 300
					BPM = 300
				Case Else
					BPM = beatsPerMinute
			End Select
			
		Else
			
			' bpm increment or decrement amount
			Select Case True
				Case beatsPerMinute < 0
					BPM = 1
				Case beatsPerMinute > 10
					BPM = 10
				Case Else
					BPM = beatsPerMinute
			End Select			
			
		End If
		
		' MIDI channel
		If Channel < 1 or Channel > 16 Then
			MIDIChannel = 1
		Else
			MIDIChannel = Channel
		End If
		
		' the preset name to be used in BandHelper
		Select Case ChangeFunction
			Case "INC"
				PresetName = "Tempo+" 
			Case "DEC"
				PresetName = "Tempo-"  
			Case ""
				If BPM <> 0 Then 
					PresetName = "T " & Right("0" & Trim(CStr(BPM)), 3) & " bpm"
				Else
						PresetName = "Tap Tempo"
				End If
		End Select
		
		If ChangeFunction = "" Then
		
			If BPM <> 0 Then
			
				' set MSB (CC-106) and LSB (CC-107)
				TempoMSB(0) = 106
				TempoMSB(1) = Fix( BPM / 128 )
				TempoLSB(0) = 107
				TempoLSB(1) = BPM Mod 128
			
			Else
				
				' Tap Tempo CC
				ControlChange(0) = 117
				ControlChange(1) = 127
				
			End If
			
		Else
			
			' set NRPN register for Tempo adjustment and general MIDI Data INC/DEC
			NRPNTempoMSB(0) = 99
			NRPNTempoMSB(1) = 106
			NRPNTempoLSB(0) = 98
			NRPNTempoLSB(1) = 107
			DataINC(0) = 96
			DataINC(1) = 1
			DataDEC(0) = 97
			DataDEC(1) = 1
			NRPNClearMSB(0) = 99
			NRPNClearMSB(1) = 127
			NRPNClearLSB(0) = 98
			NRPNClearLSB(1) = 127
		
		End If
				
		' define the Raw MIDI Hex String for Bandhelper to send to the BeatBuddy to set this tempo...
		
		If ChangeFunction = "" Then
			
			If BPM <> 0 Then
					
				' MSB raw
				RawMIDI = "B" & Hex(MIDIChannel - 1) & " " & _
				          Right("0" & Hex(TempoMSB(0)), 2) & " " & _
				          Right("0" & Hex(TempoMSB(1)), 2) & " "

				' LSB raw
				RawMIDI = RawMIDI & _
				          "B" & Hex(MIDIChannel - 1) & " " & _
				          Right("0" & Hex(TempoLSB(0)), 2) & " " & _
				          Right("0" & Hex(TempoLSB(1)), 2)
			
			Else
				
				' Tap Tempo raw CC
				RawMIDI = "B" & Hex(MIDIChannel - 1) & " " & _
				          Right("0" & Hex(ControlChange(0)), 2) & " " & _
				          Right("0" & Hex(ControlChange(1)), 2) & " "
				
			End If
					          
		Else
			
			' NRPN set
			RawMIDI = "B" & Hex(MIDIChannel - 1) & " " & _
	          		Right("0" & Hex(NRPNTempoMSB(0)), 2) & " " & _
				        Right("0" & Hex(NRPNTempoMSB(1)), 2) & " " & _
								"B" & Hex(MIDIChannel - 1) & " " & _
				        Right("0" & Hex(NRPNTempoLSB(0)), 2) & " " & _
				        Right("0" & Hex(NRPNTempoLSB(1)), 2) & " " 
			
			' increment or decrement the required number of times	        
			If ChangeFunction = "INC"	 Then		
				for i = 1 to BPM
					RawMIDI = RawMIDI & _
				  	        "B" & Hex(MIDIChannel - 1) & " " & _
				    	      Right("0" & Hex(DataINC(0)), 2) & " " & _
				      	    Right("0" & Hex(DataINC(1)), 2)	& " "
				next
			Else
				for i = 1 to BPM
					RawMIDI = RawMIDI & _
				  	        "B" & Hex(MIDIChannel - 1) & " " & _
				    	      Right("0" & Hex(DataDEC(0)), 2) & " " & _
				      	    Right("0" & Hex(DataDEC(1)), 2)	& " "
				next				
			End If
			
			' NRPN clear
			RawMIDI = RawMIDI & _
								"B" & Hex(MIDIChannel - 1) & " " & _
	          		Right("0" & Hex(NRPNClearMSB(0)), 2) & " " & _
				        Right("0" & Hex(NRPNClearMSB(1)), 2) & " " & _
								"B" & Hex(MIDIChannel - 1) & " " & _
				        Right("0" & Hex(NRPNClearLSB(0)), 2) & " " & _
				        Right("0" & Hex(NRPNClearLSB(1)), 2)  
						
		End If
		
		' return the object
		Set Init = Me
		
	End Function
  	
End Class

' BeatBuddy SongFolder class
Class SongFolder

	Public Id
	Public Description
	Public Name 	
	PUblic Number
	Public Songs()
	Public Count
	Public Path
	Public BankSelectMSB
	Public BankSelectLSB
		
	' class constructor
	Public Default Function Init( SongFolderName, SongFolderDescription, SongFolderPath, Numbering )
		
		Dim fso, inputFile	
		Dim fields	
		
		' Song Folder Id and Description as shown on the BeatBuddy menu
		Id = SongFolderName
		Description = SongFolderDescription
		
		' Path
		Path = SongFolderPath & "/" & ID		
		
		' Folder Number
		Number = CInt(Split(Description, ".")(0))
		
		' Song name  
		Name = Trim( Split(Description, ".")(1) )	
		
		' Bank Select MSB and LSB
		BankSelectMSB = Fix( Number / Numbering )
		BankSelectLSB = (Number Mod Numbering) - 1
		
		' open the Songs config.csv ile
		Set fso = CreateObject( "Scripting.FileSystemObject" )
		Set inputFile = fso.OpenTextFile( Path & "/config.csv", ForReading, True ) 
	
		' loop through each row of the config.csv file and add Song to array		
		Count = 0 
		Redim Songs(Count)
		Do Until inputFile.AtEndOfStream  				
			
			' split the row of the csv file, read into an array of fields
			fields = Split( inputFile.Readline, "," )
			
			' add new song to song array of the current song folder
			ReDim Preserve Songs(Count)
			Set Songs(Count) = (new Song)( Name, Number, fields(0), fields(1), Numbering )									
					
			Count = Count + 1  
						
		Loop
		
		' close Songs config.csv
		inputFile.Close
		Set inputFile = Nothing
		Set fso = Nothing		
		
		' return the object
		Set Init = Me
		
	End Function
		
End Class

' Song class
Class Song

	Public Id
	Public Description
	Public Name 	
	Public Number	
	Public ProgramChange			
	Public PresetName		

	' class constructor
	Public Default Function Init( SongFolderName, SongFolderNumber, SongFileName, SongDescription, Numbering )
	
		' Song Id and Description as shown on the BeatBuddy menu, plus MIDI device
		Id = Left(SongFileName, Len(SongFileName) - 4)
		Description = SongDescription		
		
		' Song name  
		Name = Trim( Split(Description, ".")(1) )		    
				
		' song number and program change value
		Number = CInt(Split(Description, ".")(0))
		ProgramChange = Number
		if Numbering = 127 then ProgramChange = ProgramChange - 1
		
		' the preset name to be used in BandHelper
		PresetName = "S " & Right("00" & Trim(CStr( SongFolderNumber )), 3) & "." & Right("00" & Trim(CStr(Number)),3) & " " & SongFolderName & ">" & Name		
					
		' return the object
		Set Init = Me
		
	End Function

End Class


'-----------------------------------------------------------------------------------------------------------------
' Functions 
'-----------------------------------------------------------------------------------------------------------------

Function BeatBuddyProjectFileExists( myFolderPath, myProjectFile )

		' defien variables needed
		Dim fso, folder, file

		' set the default response to false (i.e. BeatBuddy project file not found)
		BeatBuddyProjectFileExists = False		 
		
    ' open the project folder		
		Set fso = CreateObject( "Scripting.FileSystemObject" )
		Set folder = fso.GetFolder( myFolderPath )

		' look through al lthe files in the project folder until the BeatBuddy project file is found, when it is return true
		For each file in folder.Files			
			If file.Type = "BeatBuddy Project file" Then				
				set myProjectFile = file
				BeatBuddyProjectFileExists = True		       
				Exit For
			End If		 
		Next 		 
		 
End Function

Function SelectFolder( myStartFolder )

	' define variables
	Dim objFolder, objItem, objShell

	' Create a dialog object
	Set objShell  = CreateObject( "Shell.Application" )
	Set objFolder = objShell.BrowseForFolder( 0, "Select Folder", 0, myStartFolder )

	' Return the path of the selected folder
	' If IsObject( objFolder ) Then retval = objFolder.Self.Path
	If Not objFolder Is Nothing Then 
		SelectFolder = objFolder.Self.Path
	Else
		SelectFolder = null
	End If

	' Standard housekeeping
	Set objFolder = Nothing
	Set objshell  = Nothing

End Function

Function GetDrumSets( myProjectFolder, Channel, Numbering )    

	' define variables needed	
	Dim fso, inputFile
	Dim fields	
	Dim dsArray()
	Dim i

	' open the DrumSet config.csv ile
	Set fso = CreateObject( "Scripting.FileSystemObject" )
	Set inputFile = fso.OpenTextFile( myProjectFolder & "/DRUMSETS/config.csv", ForReading, True ) 
	
	' loop through each row of the config.csv file
	i = 0		
	ReDim dsArray(i)	
	Do Until inputFile.AtEndOfStream  
		
		' split the row of the csv file, read into an array of fields
		fields = Split( inputFile.Readline, "," )
		
		' create a new drumset object from the drumset filename and its description (field 0 and field 1 respectively) and add it to the drum set array
		ReDim Preserve dsArray(i)		    
		Set dsArray(i) = (new DrumSet)( fields(0), fields(1), Channel, Numbering )								
		i = i + 1  
		
	Loop
	
	' close the config.csv file
	inputFile.Close
	Set inputFile = Nothing
	Set fso = Nothing
	
	' return the array of drumsets
	GetDrumSets = dsArray	

End Function

Function GetSongFolders( myProjectFolder, midiNumbering )    

	' define variables needed	
	Dim fso, inputSongFolderFile, inputSongsFile
	Dim fields	
	Dim sngFolderArray()
	Dim i

	' open the Song Folder config.csv ile
	Set fso = CreateObject( "Scripting.FileSystemObject" )
	Set inputSongFolderFile = fso.OpenTextFile( myProjectFolder & "/SONGS/config.csv", ForReading, True ) 
	
	' loop through each row of the Song Folder config.csv file
	i = 0		
	ReDim sngFolderArray(i)	
	Do Until inputSongFolderFile.AtEndOfStream  
		
		' split the row of the csv file, read into an array of fields
		fields = Split( inputSongFolderFile.Readline, "," )
		
		' create a new song folder object from the folder name and its description (field 0 and field 1 respectively) and add to song folder array
		' a songl folde rincludes the songs within it		
		ReDim Preserve sngFolderArray(i)		    		
		Set sngFolderArray(i) = (new SongFolder)( fields(0), fields(1), myProjectFolder & "/SONGS", midiNumbering )			
		i = i + 1  
		
	Loop
	
	' close the config.csv file
	inputSongFolderFile.Close
	Set inputSongFolderFile = Nothing
	Set fso = Nothing
	
	' return the array of drumsets
	GetSongFolders = sngFolderArray	

End Function

Sub OKorQuit( test )
	If vartype(test) = vbEmpty or vartype(test) = vbNull Then 
		On Error Resume Next
		outputFile.Close
		fso.DeleteFile tempOutputFileName 
		Set outputFile = Nothing
		Set fso = Nothing
		On Error GoTo 0
		WScript.Quit(-1)
	Else
		test = Trim(test)
	End If	
End Sub

'-----------------------------------------------------------------------------------------------------------------
' Main Script Body
'-----------------------------------------------------------------------------------------------------------------

' constants required
Const ForReading = 1
Const ForWriting = 2
Const ForAppending = 8
	
' variable definion
Dim WshShell
Dim appTitle, myMidiDevice, midiChannel, presetPrefix, midiNumbering
Dim projectFolderPath, currentPath, projectFile, projectName
Dim BeatBuddyDrumSet, myDrumSets
Dim BeatBuddySongFolder, BeatBuddySong, mySongFolders
Dim fromTempo, toTempo, incTempo, myTempo, newTempo, tempoInputError, myMidiChannel, stepBPM
Dim fso, outputFile, tempOutputFileName, OutputFileName
Dim DrumSetCount, SongFolderCount, SongCount, TempoCount
Dim responseStr, tmpStr1, tmpStr2, tmpStr3

' default values
myMidiDevice = "BeatBuddy"
appTitle = "Bandhelper/BeatBuddy MIDI Preset Builder"
presetPrefix = ""
DrumSetCount = 0
SongFolderCount = 0
SongCount = 0
TempoCount = 0
responseStr = ""
midiNumbering = 127

' Select BeatBuddy Project folder
Set fso = CreateObject( "Scripting.FileSystemObject" )
currentPath = fso.GetAbsolutePathName(".")
projectFolderPath = SelectFolder( currentPath  )
OKorQuit projectFolderPath

' Check selected folder actually contains a BeatBuddy project
If BeatBuddyProjectFileExists ( projectFolderPath, projectFile ) Then

	' project name from BBP file name
	projectName = Left( projectFile.Name, Len(projectFile.Name) - 4 )

	' open/create importMidi.txt
	tempOutputFileName = fso.GetTempName
	OutputFileName = currentPath & "/" & projectName  & " - importMidi.txt"
	Set outputFile = fso.OpenTextFile( currentPath & "/" & tempOutputFileName , ForWriting, True ) 

	' check if user would like to prefix project sepcific MIDI preset names with the project name	
	presetPrefix = InputBox( "Please provide a prefix value for any project specific MIDI presets:-", appTitle, projectName )	
	OKorQuit presetPrefix
	if presetPrefix <> "" then presetPrefix = presetPrefix & " "

	' prompt user for name of the BeatBuddy MIDI device in Bandhelper
	Do
		myMidiDevice = InputBox( "Please enter the MIDI device name of your BeatBuddy as it appears to the Bandhelper app (case-sensitive):-", appTitle, myMidiDevice )
		OKorQuit myMidiDevice
	Loop Until myMidiDevice <> ""

	' prompt user to see what the numbering setting is in Bandhelper for the BeatBuddy device (MIDI preset need to be defined slighlty differently depending on this)
	' The BeatBuddy manual says to subtract 1 from your folder and song numbers, because MIDI values are often counted starting from 0 thru 127. However, 
	' this isn't necessary in Bandhelper, thanks to the 1-128 setting that can selected in the device setup. This is more intuitive since it also matches 
	' the numbers you see displayed on the BeatBuddy screen
	If MsgBox( "Have you set the Numbering option of the BeatBuddy within the BandHelper MIDI Device settings to 1-128?", vbYesNo, appTitle ) = vbYes Then
		midiNumbering = 128
	End If	

	' --- Drum Sets ---

				' check if user would like to include Drum Sets
				If MsgBox( "Would you like to include Drum Set MIDI presets?", vbYesNo, appTitle ) = vbYes Then

					' allow user to select MIDI channel for Drum Set change
					On Error Resume Next		
					Do
						responseStr = InputBox( "What MIDI channel do you wish to send the Drum Set change to the BeatBuddy on?", appTitle, "1" )
						OKorQuit responseStr
						myMidiChannel = CInt( responseStr )
					Loop Until err.Number = 0 And myMidiChannel >= 1 And myMidiChannel <= 16
					On Error GoTo 0
					
					' get an array of the drumsets found in the project, including the RAW midi Hex codes to select them
					myDrumSets = GetDrumSets( projectFolderPath, myMidiChannel, midiNumbering )	
								
					' loop through the DrumSets and write MIDI presets to file
					For each BeatBuddyDrumSet in myDrumSets 		 
						 outputFile.WriteLine presetPrefix & BeatBuddyDrumSet.PresetName & vbTab & vbTab & myMidiDevice & vbTab & BeatBuddyDrumSet.RawMIDI & vbTab 
					Next
					DrumSetCount = UBound(myDrumSets) + 1
					
				End If
	
	' --- Songs ---

				' check if user would like to include Songs
				If MsgBox( "Would you like to include Song MIDI presets?", vbYesNo, appTitle ) = vbYes Then
					
					' get an array of song folders and song content in the project
					mySongFolders = GetSongFolders( projectFolderPath, midiNumbering )	
								
					' loop through the Song folders to get songs and write out MIDI presets to file
					For each BeatBuddySongFolder in mySongFolders
					
						For each BeatBuddySong in BeatBuddySongFolder.Songs
							outputFile.WriteLine presetPrefix & BeatBuddySong.PresetName & vbTab & myMidiDevice & "|" & BeatBuddySongFolder.BankSelectMSB & "|" & BeatBuddySongFolder.BankSelectLSB & "|" & BeatBuddySong.ProgramChange & vbTab & vbTab & vbTab 							
						Next
						SongCount = SongCount + BeatBuddySongFolder.Count
						
					Next	
					SongFolderCount = UBound(mySongFolders) + 1
																				
				End If
	
	' --- Tempos ---
	
				' check if user would like to include Tempo Presets
				If MsgBox( "Would you like to include a range of Tempo MIDI presets?", vbYesNo, appTitle ) = vbYes Then
					
					' get the tempo range and increment between presets required from the user
					On Error Resume Next		
					Do
						responseStr =  InputBox( "What MIDI channel do you wish to send the tempo change to the BeatBuddy on?", appTitle, "1" )	
						OKorQuit responseStr
						myMidiChannel = CInt( responseStr )
					Loop Until err.Number = 0 And myMidiChannel >= 1 And myMidiChannel <= 16
					Do
						responseStr = InputBox( "What is the slowest tempo (bpm) you would like to start at?", appTitle, "40" )
						OKorQuit responseStr
						fromTempo = CInt( responseStr )						
					Loop Until err.Number = 0 And fromTempo >= 40 And fromTempo <= 300
					Do
						responseStr = InputBox( "What is the fastest tempo (bpm) you would like to go up to?", appTitle, "300" )
						OKorQuit responseStr
						toTempo = CInt( responseStr )												
					Loop Until err.Number = 0 And toTempo >= 40 And toTempo <= 300 and toTempo >= fromTempo
					' MsgBox "A"
					Do
						responseStr = InputBox( "What is the bpm increment you would like between tempo presets?", appTitle, "2" )
						OKorQuit responseStr
						incTempo = CInt( responseStr )
					Loop Until err.Number = 0 And incTempo >= 1 
					' MsgBox "B"
					On Error GoTo 0
					
					On Error Resume Next		
					Do
						responseStr =  InputBox( "For simple Tempo+ and Tempo- presets what is your prefered increment/decrement bpm step?", appTitle, "1" )	
						OKorQuit responseStr
						stepBPM = CInt( responseStr )
					Loop Until err.Number = 0 And stepBPM > 0 And stepBPM <= 10
					On Error GoTo 0
					
					' MsgBox "1"
					' loop through required tempo range and output presets
					For myTempo = fromTempo to toTempo Step incTempo
						Set  newTempo = (new Tempo)( myTempo, myMidiChannel, "" )						
						TempoCount = TempoCount + 1
						outputFile.WriteLine newTempo.PresetName & vbTab & vbTab & myMidiDevice & vbTab & newTempo.RawMIDI & vbTab 
					Next					
					' MsgBox "2"
					' Create preset for Tempo Increment
					Set  newTempo = (new Tempo)( stepBPM, myMidiChannel, "INC" )						
					TempoCount = TempoCount + 1
					outputFile.WriteLine newTempo.PresetName & vbTab & vbTab & myMidiDevice & vbTab & newTempo.RawMIDI & vbTab 
					' MsgBox "3"
					' Create preset for Tempo Increment					
					Set  newTempo = (new Tempo)( stepBPM, myMidiChannel, "DEC" )						
					TempoCount = TempoCount + 1
					outputFile.WriteLine newTempo.PresetName & vbTab & vbTab & myMidiDevice & vbTab & newTempo.RawMIDI & vbTab 
					' MsgBox "4"
					'Create preset for Tap Tempo
					Set  newTempo = (new Tempo)( 0, myMidiChannel, "" )						
					TempoCount = TempoCount + 1
					outputFile.WriteLine newTempo.PresetName & vbTab & vbTab & myMidiDevice & vbTab & newTempo.RawMIDI & vbTab 
					' MsgBox "5"
							
				End If

	' close the importMidi.txt file and rename temp file
	outputFile.Close
	fso.CopyFile tempOutputFileName, OutputFileName, true
	fso.DeleteFile tempOutputFileName 
	Set outputFile = Nothing
	Set fso = Nothing
	
	' display success and quit
	if DrumSetCount = 1 then tmpStr1 = "preset" Else tmpStr1 = "presets" End If
	if SongCount = 1 then tmpStr2 = "preset" Else tmpStr2 = "presets" End If
	if SongFolderCount > 1 Then tmpStr2 = tmpStr2 & " from " & SongFolderCount & " folders" End If
	If TempoCount = 1 then tmpStr3 = "preset" Else tmpStr3 = "presets"  End If	
	MsgBox "importMIDI.txt file created successfully containing " & DrumSetCount & " drum set " & tmpStr1 & ", " & SongCount & " song " & tmpStr2 & " and " & TempoCount & " tempo " & tmpStr3 & ".", vbOKOnly, appTitle & ": Success!"
	WScript.Quit(0)

Else
	MsgBox "No BeatBuddy project found within the selected folder.", vbOKOnly, appTitle & ": Error"
	WScript.Quit(1)
End If

' end of script